import { resolveAction, } from '../../vendors/organik';
import { getAppHelperForInternalUse } from '../../vendors/organik/core/app';
import { isPlainObject } from '../../utils';
import { getDefaultActionNames, resolveActionsFromView } from '../../utils/widget';
import { ViewHeadlessWidget } from './View';
class ObjectViewHeadlessWidget extends ViewHeadlessWidget {
    getDefaultActions() {
        return this.getCommonBehavior('view.objectViewDefaultActions');
    }
    resolveActionNodes(defaultActionsGetter, nodeCreator, executors = {}, actionText = {}, readonly) {
        const context = this.getViewContext();
        const config = this.getConfig();
        const certainActions = resolveActionsFromView(context);
        const defaultExecutors = {};
        const resolvedActionText = {};
        getDefaultActionNames('submitAction').forEach(actionName => {
            defaultExecutors[actionName] = () => context.submit();
            if (config.submitActionText) {
                resolvedActionText[actionName] = config.submitActionText;
            }
            else if (actionText.submitActionText) {
                resolvedActionText[actionName] = actionText.submitActionText;
            }
        });
        getDefaultActionNames('resetAction').forEach(actionName => {
            defaultExecutors[actionName] = () => context.reset();
            if (config.resetActionText) {
                resolvedActionText[actionName] = config.resetActionText;
            }
            else if (actionText.resetActionText) {
                resolvedActionText[actionName] = actionText.resetActionText;
            }
        });
        getDefaultActionNames('cancelAction').forEach(actionName => {
            if (config.cancelActionText) {
                resolvedActionText[actionName] = config.cancelActionText;
            }
            else if (actionText.cancelActionText) {
                resolvedActionText[actionName] = actionText.cancelActionText;
            }
        });
        const resolvedExecutors = Object.assign(Object.assign({}, defaultExecutors), executors);
        const actionNodes = [];
        const resolvedActions = context.getActions();
        if (resolvedActions.length === 0) {
            defaultActionsGetter(readonly).forEach(refOrDescriptor => {
                const action = resolveAction(refOrDescriptor);
                if (action) {
                    resolvedActions.push(action);
                }
            });
        }
        resolvedActions.forEach(action => {
            if (!action.context || action.context === 'single') {
                const resolvedAction = Object.assign({}, action);
                if (action.name) {
                    const defaultExecutor = resolvedExecutors[action.name];
                    if (!resolvedAction.execute && defaultExecutor) {
                        resolvedAction.execute = defaultExecutor;
                    }
                    const textFromConfig = resolvedActionText[action.name];
                    if (!certainActions[action.name] && textFromConfig) {
                        resolvedAction.text = textFromConfig;
                    }
                }
                actionNodes.push(nodeCreator(resolvedAction));
            }
        });
        return actionNodes;
    }
    getRecordParams() {
        const { recordGetterRouteParams = 'id' } = this.getConfig();
        const routeParams = getAppHelperForInternalUse().history.getLocation().params;
        if (isPlainObject(recordGetterRouteParams)) {
            return Object.entries(recordGetterRouteParams).reduce((params, [recordParamKey, routeParamKey]) => (Object.assign(Object.assign({}, params), { [recordParamKey]: routeParams[routeParamKey] })), {});
        }
        const keys = [].concat(recordGetterRouteParams);
        return keys.length > 1
            ? keys.reduce((params, key) => (Object.assign(Object.assign({}, params), { [key]: routeParams[key] })), {})
            : routeParams[keys[0]];
    }
    isNewOne() {
        return !this.getRecordParams() || !this.getViewContext().getOne;
    }
    fetchData(opener) {
        if (opener ? opener.getView().category !== 'object' : this.isNewOne()) {
            return;
        }
        const context = this.getViewContext();
        context.setBusy(true);
        context
            .getOne(opener ? opener.getValue() : this.getRecordParams(), data => context.setDataSource(data))
            .finally(() => context.setBusy(false));
    }
}
export { ObjectViewHeadlessWidget };
